@@ -18,6 +18,7 @@ import httpx
 import moexalgo
 import polars as pl
 
+from src.exceptions import CategoryNotFound, TickerNotFound
 from src.manager.category import Category
 from src.manager.iid import Iid
 from src.manager.iid_cache import IidCache
@@ -27,7 +28,7 @@ from src.utils import Cmd, cfg, dt_to_ts, log, now, prev_month
 
 SOURCE = Source.MOEX
 MSK_OFFSET = TimeDelta(hours=3)
-MSK_TS = 3 * 60 * 60 * 1_000_000_000  # ts_nanos offset
+MSK_OFFSET_TS = 3 * 60 * 60 * 1_000_000_000  # ts_nanos offset
 AVAILIBLE = [
     MarketData.BAR_1M,
     MarketData.BAR_10M,
@@ -54,22 +55,43 @@ class SourceMoex:
         cls.__authorizate()
 
         # moex_categories = ["index", "shares", "currency", "futures"]
-        moex_categories = ["shares"]
+        moex_categories = ["index", "shares", "futures", "currency"]
         for i in moex_categories:
-            df = cls.__request_instruments(i)
-            category = cls.__to_avin_category(i)
-            cache = IidCache(SOURCE, category, df)
+            try:
+                df = cls.__request_instruments(i)
+                category = cls.__to_avin_category(i)
+                cache = IidCache(SOURCE, category, df)
+                IidCache.save(cache)
 
-            IidCache.save(cache)
+            except httpx.ConnectError as e:
+                log.warning(f"ConnectError: {e}. Try again after 5 sec")
+                time.sleep(5)
+
+            except httpx.ConnectTimeout as e:
+                log.warning(f"ConnectTimeout: {e}. Try again after 5 sec")
+                time.sleep(5)
 
     @classmethod
     def find(
         cls,
         s: str,
-    ) -> Iid | None:
+    ) -> Iid:
+        """Get instrument id from str.
+
+        Args:
+            s: search querry. Example: "moex_share_sber", "moex_index_imoex2".
+
+        Returns:
+            Iid.
+
+        Raises:
+            TickerNotFound if not exists.
+        """
         # parse str
         exchange_str, category_str, ticker_str = s.upper().split("_")
-        assert exchange_str == "MOEX"
+        assert exchange_str == "MOEX", (
+            f"Not supported {exchange_str}, only MOEX"
+        )
         category = Category.from_str(category_str)
 
         # load cache
@@ -79,33 +101,33 @@ class SourceMoex:
         # try find ticker
         df = df.filter(pl.col("ticker") == ticker_str)
         if len(df) != 1:
-            return None
+            raise TickerNotFound(f"Cannot find ticker {ticker_str}")
 
-        # NOTE: MOEX not provide figi... using unique fake value
         match category:
             case Category.SHARE:
-                info = {
-                    "exchange": "MOEX",
-                    "category": category_str,
-                    "ticker": ticker_str,
-                    "figi": f"figi_MOEX_{category_str}_{ticker_str}",
-                    "name": df.item(0, "shortname"),
-                    "lot": df.item(0, "lotsize"),
-                    "step": df.item(0, "minstep"),
-                }
+                lotsize = df.item(0, "lotsize")
+                step = df.item(0, "minstep")
             case Category.FUTURE:
-                info = {
-                    "exchange": "MOEX",
-                    "category": category_str,
-                    "ticker": ticker_str,
-                    "figi": f"figi_MOEX_{category_str}_{ticker_str}",
-                    "name": df.item(0, "shortname"),
-                    "lot": 1,
-                    "step": df.item(0, "minstep"),
-                }
+                lotsize = 1
+                step = df.item(0, "minstep")
+            case Category.INDEX:
+                lotsize = 1
+                step = 10 ** -float(df.item(0, "decimals"))
             case _:
-                log.error(f"Not implemented category {category_str}, {df}")
-                exit(1)
+                raise CategoryNotFound(
+                    f"Not implemented category {category_str}, {df}"
+                )
+
+        # NOTE: MOEX not provide figi... using unique fake value
+        info = {
+            "exchange": exchange_str,
+            "category": category_str,
+            "ticker": ticker_str,
+            "figi": f"figi_MOEX_{category_str}_{ticker_str}",
+            "name": df.item(0, "shortname"),
+            "lot": lotsize,
+            "step": step,
+        }
 
         return Iid(info)
 
@@ -199,23 +221,19 @@ class SourceMoex:
         response = market.tickers(use_dataframe=True)
         df = pl.from_pandas(response)
 
-        df = df.with_columns(
-            pl.col("lotsize").cast(pl.String).alias("lotsize")
-        )
-        df = df.with_columns(
-            pl.col("decimals").cast(pl.String).alias("decimals")
-        )
-        df = df.with_columns(
-            pl.col("minstep").cast(pl.String).alias("minstep")
-        )
-        df = df.with_columns(
-            pl.col("issuesize").cast(pl.String).alias("issuesize")
-        )
-        df = df.with_columns(
-            pl.col("listlevel").cast(pl.String).alias("listlevel")
-        )
+        match moex_category:
+            case "index":
+                return _format_indexes_info(df)
+            case "shares":
+                return _format_shares_info(df)
+            case "futures":
+                return _format_futures_info(df)
+            case "currency":
+                return _format_currencies_info(df)
 
-        return df
+        raise CategoryNotFound(
+            f"Not implemented category {moex_category}, {df}"
+        )
 
     @classmethod
     def __to_avin_category(cls, name: str) -> Category:
@@ -247,13 +265,11 @@ class SourceMoex:
                 return moex_ticker
 
             except httpx.ConnectError as e:
-                log.warning(f"ConnectError: {e}")
-                log.info("Try again after 3 sec")
+                log.warning(f"ConnectError: {e}. Try again after 3 sec")
                 time.sleep(3)
 
             except httpx.ConnectTimeout as e:
-                log.warning(f"ConnectTimeout: {e}")
-                log.info("Try again after 3 sec")
+                log.warning(f"ConnectTimeout: {e}. Try again after 3 sec")
                 time.sleep(3)
 
             attempt += 1
@@ -359,7 +375,7 @@ class SourceMoex:
             current = bars.item(-1, "begin")
 
         # format df
-        df = cls.__format_bars_df(bars)
+        df = _format_bars_df(bars)
 
         return df
 
@@ -385,13 +401,11 @@ class SourceMoex:
                 return pl.from_pandas(df)
 
             except httpx.ConnectError as e:
-                log.warning(f"ConnectError: {e}")
-                log.info("Try again after 3 sec")
+                log.warning(f"ConnectError: {e}. Try again after 3 sec")
                 time.sleep(3)
 
             except httpx.ConnectTimeout as e:
-                log.warning(f"ConnectTimeout: {e}")
-                log.info("Try again after 3 sec")
+                log.warning(f"ConnectTimeout: {e}. Try again after 3 sec")
                 time.sleep(3)
 
             attempt += 1
@@ -399,22 +413,6 @@ class SourceMoex:
         log.error("Request data failed")
         exit(1)
 
-    @classmethod
-    def __format_bars_df(cls, bars: pl.DataFrame) -> pl.DataFrame:
-        df = pl.DataFrame(
-            {
-                "ts_nanos": bars["begin"].cast(pl.Int64) - MSK_TS,
-                "open": bars["open"],
-                "high": bars["high"],
-                "low": bars["low"],
-                "close": bars["close"],
-                "volume": bars["volume"].cast(pl.Int64),
-                "value": bars["value"],
-            }
-        )
-
-        return df
-
     @classmethod
     def __get_tics(
         cls,
@@ -444,7 +442,7 @@ class SourceMoex:
             tics.extend(part)
 
         # format df
-        df = cls.__format_tics_df(tics)
+        df = _format_tics_df(tics)
 
         return df
 
@@ -466,13 +464,11 @@ class SourceMoex:
                 return pl.from_pandas(df)
 
             except httpx.ConnectError as e:
-                log.warning(f"ConnectError: {e}")
-                log.info("Try again after 3 sec")
+                log.warning(f"ConnectError: {e}. Try again after 3 sec")
                 time.sleep(3)
 
             except httpx.ConnectTimeout as e:
-                log.warning(f"ConnectTimeout: {e}")
-                log.info("Try again after 3 sec")
+                log.warning(f"ConnectTimeout: {e}. Try again after 3 sec")
                 time.sleep(3)
 
             attempt += 1
@@ -480,31 +476,106 @@ class SourceMoex:
         log.error("Request data failed")
         exit(1)
 
-    @classmethod
-    def __format_tics_df(cls, tics: pl.DataFrame) -> pl.DateFrame:
-        # convert date & time to timestamp
-        day = Date.today()
-        times = tics["tradetime"]
-        timestamps = list()
-        for t in times:
-            dt = DateTime.combine(day, t, tzinfo=UTC) - MSK_OFFSET
-            ts = dt_to_ts(dt)
-            timestamps.append(ts)
 
-        # format dataframe
-        df = pl.DataFrame(
-            {
-                "ts_nanos": timestamps,
-                "direction": tics["buysell"],
-                "lots": tics["quantity"],
-                "price": tics["price"],
-                "value": tics["value"],
-                "session": tics["tradingsession"].cast(pl.Int8),
-                "tradeno": tics["tradeno"],
-            }
-        )
+def _format_indexes_info(df: pl.DataFrame) -> pl.DataFrame:
+    columns = ["decimals"]
+    for name in columns:
+        df = df.with_columns(pl.col(name).cast(pl.String).alias(name))
 
-        return df
+    return df
+
+
+def _format_shares_info(df: pl.DataFrame) -> pl.DataFrame:
+    columns = ["lotsize", "decimals", "minstep", "issuesize", "listlevel"]
+    for name in columns:
+        df = df.with_columns(pl.col(name).cast(pl.String).alias(name))
+
+    return df
+
+
+def _format_futures_info(df: pl.DataFrame) -> pl.DataFrame:
+    columns = ["decimals", "minstep"]
+    for name in columns:
+        df = df.with_columns(pl.col(name).cast(pl.String).alias(name))
+
+    return df
+
+
+def _format_currencies_info(df: pl.DataFrame) -> pl.DataFrame:
+    columns = ["lotsize", "decimals", "minstep"]
+    for name in columns:
+        df = df.with_columns(pl.col(name).cast(pl.String).alias(name))
+
+    return df
+
+
+def _format_bars_df(bars: pl.DataFrame) -> pl.DataFrame:
+    df = pl.DataFrame(
+        {
+            "ts_nanos": bars["begin"].cast(pl.Int64) - MSK_OFFSET_TS,
+            "open": bars["open"],
+            "high": bars["high"],
+            "low": bars["low"],
+            "close": bars["close"],
+            "volume": bars["volume"].cast(pl.Int64),
+            "value": bars["value"],
+        }
+    )
+
+    return df
+
+
+def _format_tics_df(tics: pl.DataFrame) -> pl.DateFrame:
+    # format dataframe
+    df = pl.DataFrame(
+        {
+            "ts_nanos": _get_safely(tics, "ts_nanos"),
+            "direction": _get_safely(tics, "direction"),
+            "lots": _get_safely(tics, "lots"),
+            "price": tics["price"],
+            "value": _get_safely(tics, "value"),
+            "session": _get_safely(tics, "session"),
+            "tradeno": _get_safely(tics, "tradeno"),
+        }
+    )
+
+    return df
+
+
+def _get_safely(df: pl.DataFrame, key: str) -> pl.Series | None:
+    """Get value (pl.Series) or None if key not exists."""
+
+    match key:
+        case "ts_nanos":
+            # convert date & time to timestamp
+            day = Date.today()
+            times = df["tradetime"]
+            timestamps = list()
+            for t in times:
+                dt = DateTime.combine(day, t, tzinfo=UTC) - MSK_OFFSET
+                ts = dt_to_ts(dt)
+                timestamps.append(ts)
+            return pl.Series("ts_nanos", timestamps)
+        case "direction":
+            if "buysell" in df.columns:
+                return df["buysell"]
+        case "lots":
+            if "quantity" in df.columns:
+                return df["quantity"]
+        case "tradeno":
+            if "tradeno" in df.columns:
+                return df["tradeno"]
+        case "session":
+            if "tradingsession" in df.columns:
+                return df["tradingsession"].cast(pl.Int8)
+        case "value":
+            if "value" in df.columns:
+                return df["value"]
+            else:
+                # это фьючерс... вручную умножим lotsize=1 * lots * price
+                return df["quantity"] * df["price"]
+
+    return None
 
 
 if __name__ == "__main__":
