@@ -10,18 +10,33 @@ from __future__ import annotations
 
 import enum
 
+from src.exceptions import SourceNotFound
+
 
 class Source(enum.Enum):
+    """Market data source."""
+
     MOEX = 1
     TINKOFF = 2
 
     @classmethod
     def from_str(cls, string: str) -> Source:
-        sources = {
-            "MOEX": Source.MOEX,
-            "TINKOFF": Source.TINKOFF,
-        }
-        return sources[string.upper()]
+        """Get enum from str.
+
+        Args:
+            string: category name.
+
+        Returns:
+            Category Enum.
+
+        Raises:
+            SourceNotFound if category not exists.
+        """
+        if attr := getattr(cls, string.upper(), None):
+            return attr
+        raise SourceNotFound(
+            f"Source not found. Choice from {Source._member_names_}"
+        )
 
 
 if __name__ == "__main__":
