@@ -1,19 +1,23 @@
 #!/usr/bin/env  python3
 # ============================================================================
-# URL:          http://arsvincere.com
+# URL:          http://avin.info
 # AUTHOR:       Alex Avin
 # E-MAIL:       mr.alexavin@gmail.com
 # LICENSE:      MIT
 # ============================================================================
 
+import sys
+from datetime import datetime as DateTime
+
+sys.path.append("/home/alex/avin/avin_data")
 from avin_data import *
 
 
 def test_utc_dt():
-    dt = utc("2024-01-02")
+    dt = str_to_utc("2024-01-02")
     assert str(dt) == "2024-01-01 21:00:00+00:00"
 
-    dt = utc("2025-01-01 10:00")
+    dt = str_to_utc("2025-01-01 10:00")
     assert str(dt) == "2025-01-01 07:00:00+00:00"
 
 
